@@ -11,25 +11,28 @@ import os
 import sys
 import warnings
 
-from quodlibet.compat import builtins, PY2
+from quodlibet.compat import PY2
 
 if PY2:
     # some code depends on utf-8 default encoding (pygtk used to set it)
     reload(sys)
     sys.setdefaultencoding("utf-8")
 
+from quodlibet import util
 from quodlibet.util import set_process_title, environ, cached_func
-from quodlibet.util import windows, is_osx
+from quodlibet.util import windows, is_osx, is_windows
 from quodlibet.util.path import mkdir, unexpand
 from quodlibet.util.i18n import GlibTranslations, set_i18n_envvars, \
     fixup_i18n_envvars
-from quodlibet.util.dprint import print_, print_d, print_w, print_e
+from quodlibet.util.dprint import print_d, print_e
 from quodlibet import const
-from quodlibet.const import MinVersions, Version
+from quodlibet import build
+from quodlibet.const import MinVersions
+from quodlibet.compat import PY2
 
 
 PLUGIN_DIRS = ["editing", "events", "playorder", "songsmenu", "playlist",
-               "gstreamer", "covers"]
+               "gstreamer", "covers", "query"]
 
 
 GlibTranslations().install(unicode=True)
@@ -55,6 +58,9 @@ class Application(object):
     library = None
     player = None
 
+    player_options = None
+    """A PlayerOptions instance or None in case there is no playback support"""
+
     cover_manager = None
 
     name = None
@@ -63,6 +69,14 @@ class Application(object):
     id = None
     """The application ID e.g. 'quodlibet'"""
 
+    @property
+    def icon_name(self):
+        return self.id
+
+    @property
+    def symbolic_icon_name(self):
+        return "%s-symbolic" % self.icon_name
+
     @property
     def librarian(self):
         return self.library.librarian
@@ -110,14 +124,42 @@ def is_release():
     return const.VERSION_TUPLE[-1] != -1
 
 
+def get_build_version():
+    """Returns a build version tuple"""
+
+    version = list(const.VERSION_TUPLE)
+    if version[-1] != -1 and build.BUILD_VERSION > 0:
+        version.append(build.BUILD_VERSION)
+
+    return tuple(version)
+
+
+def get_build_description():
+    """Returns text describing the version of the build.
+
+    Includes additional build info like git hash and build version.
+    """
+
+    version = list(get_build_version())
+    notes = []
+    if version[-1] == -1:
+        version = version[:-1]
+        notes.append(u"development")
+
+    if build.BUILD_INFO:
+        notes.append(build.BUILD_INFO)
+
+    version_string = u".".join(map(str, version))
+    note = u" (%s)" % u", ".join(notes) if notes else u""
+
+    return version_string + note
+
+
 @cached_func
 def get_base_dir():
     """The path to the quodlibet package"""
 
-    file_path = __file__
-    if os.name == "nt":
-        file_path = file_path.decode(sys.getfilesystemencoding())
-    return os.path.dirname(os.path.realpath(file_path))
+    return util.get_module_dir()
 
 
 @cached_func
@@ -136,19 +178,14 @@ def get_user_dir():
     else:
         USERDIR = os.path.join(os.path.expanduser("~"), ".quodlibet")
 
+    if not PY2:
+        USERDIR += "_py3"
+
     if 'QUODLIBET_USERDIR' in environ:
         USERDIR = environ['QUODLIBET_USERDIR']
 
-    # XXX: Exec conf.py in this directory, used to override const globals
-    # e.g. for setting USERDIR for the Windows portable version
-    # Note: execfile doesn't handle unicode paths on windows, so encode.
-    # (this doesn't use the old win api in case of str compared to os.*)
-    _CONF_PATH = os.path.join(
-        os.path.dirname(os.path.realpath(__file__)), "conf.py")
-    try:
-        execfile(_CONF_PATH)
-    except IOError:
-        pass
+    if build.BUILD_TYPE == u"windows-portable":
+        USERDIR = os.path.join(get_base_dir(), "..", "..", "..", "config")
 
     # XXX: users shouldn't assume the dir is there, but we currently do in
     # some places
@@ -193,11 +230,53 @@ def _fix_gst_leaks():
         Gst.Element.add_pad = do_wrap(Gst.Element.add_pad)
 
 
+def _init_g():
+    """Call before using GdkPixbuf/GLib/Gio/GObject"""
+
+    import gi
+
+    gi.require_version("GLib", "2.0")
+    gi.require_version("Gio", "2.0")
+    gi.require_version("GObject", "2.0")
+    gi.require_version("GdkPixbuf", "2.0")
+
+    from gi.repository import GdkPixbuf
+
+    # On windows the default variants only do ANSI paths, so replace them.
+    # In some typelibs they are replaced by default, in some don't..
+    if os.name == "nt":
+        for name in ["new_from_file_at_scale", "new_from_file_at_size",
+                     "new_from_file"]:
+            cls = GdkPixbuf.Pixbuf
+            setattr(cls, name, getattr(cls, name + "_utf8", name))
+
+    # https://bugzilla.gnome.org/show_bug.cgi?id=670372
+    if not hasattr(GdkPixbuf.Pixbuf, "savev"):
+        GdkPixbuf.Pixbuf.savev = GdkPixbuf.Pixbuf.save
+
+    # Newer glib is noisy regarding deprecated signals/properties
+    # even with stable releases.
+    if is_release():
+        warnings.filterwarnings(
+            'ignore', '.* It will be removed in a future version.',
+            Warning)
+
+    # blacklist some modules, simply loading can cause segfaults
+    sys.modules["glib"] = None
+    sys.modules["gobject"] = None
+
+
 def _init_gtk():
     """Call before using Gtk/Gdk"""
 
     import gi
 
+    # pygiaio 3.14rev16 switched to fontconfig for PangoCairo. As this results
+    # in 100% CPU under win7 revert it. Maybe we need to update the
+    # cache in the windows installer for it to work... but for now revert.
+    if is_windows():
+        os.environ['PANGOCAIRO_BACKEND'] = 'win32'
+
     # disable for consistency and trigger events seem a bit flaky here
     if is_osx():
         os.environ["GTK_OVERLAY_SCROLLING"] = "0"
@@ -214,15 +293,12 @@ def _init_gtk():
     except (ValueError, ImportError):
         pass
 
-    gi.require_version("GLib", "2.0")
     gi.require_version("Gtk", "3.0")
     gi.require_version("Gdk", "3.0")
-    gi.require_version("GObject", "2.0")
     gi.require_version("Pango", "1.0")
-    gi.require_version("GdkPixbuf", "2.0")
-    gi.require_version("Gio", "2.0")
+    gi.require_version('Soup', '2.4')
 
-    from gi.repository import Gtk, GObject, Gdk, GdkPixbuf
+    from gi.repository import Gtk, Gdk
 
     # PyGObject doesn't fail anymore when init fails, so do it ourself
     initialized, argv = Gtk.init_check(sys.argv)
@@ -230,41 +306,12 @@ def _init_gtk():
         raise SystemExit("Gtk.init failed")
     sys.argv = list(argv)
 
-    # add Gtk.TreePath.__getitem__/__len__ for PyGObject 3.2
-    try:
-        Gtk.TreePath()[0]
-    except TypeError:
-        Gtk.TreePath.__getitem__ = lambda self, index: list(self)[index]
-        Gtk.TreePath.__len__ = lambda self: self.get_depth()
-
-    # GTK+ 3.4+ constants
-    if not hasattr(Gdk, "BUTTON_PRIMARY"):
-        Gdk.BUTTON_PRIMARY = 1
-        Gdk.BUTTON_MIDDLE = 2
-        Gdk.BUTTON_SECONDARY = 3
-
-    if not hasattr(Gdk, "EVENT_PROPAGATE"):
-        Gdk.EVENT_PROPAGATE = 0
-        Gdk.EVENT_STOP = 1
-
     # include our own icon theme directory
     theme = Gtk.IconTheme.get_default()
     theme_search_path = get_image_dir()
     assert os.path.exists(theme_search_path)
     theme.append_search_path(theme_search_path)
 
-    # On windows the default variants only do ANSI paths, so replace them.
-    # In some typelibs they are replaced by default, in some don't..
-    if os.name == "nt":
-        for name in ["new_from_file_at_scale", "new_from_file_at_size",
-                     "new_from_file"]:
-            cls = GdkPixbuf.Pixbuf
-            setattr(cls, name, getattr(cls, name + "_utf8", name))
-
-    # https://bugzilla.gnome.org/show_bug.cgi?id=670372
-    if not hasattr(GdkPixbuf.Pixbuf, "savev"):
-        GdkPixbuf.Pixbuf.savev = GdkPixbuf.Pixbuf.save
-
     # Force menu/button image related settings. We might show too many atm
     # but this makes sure we don't miss cases where we forgot to force them
     # per widget.
@@ -280,19 +327,13 @@ def _init_gtk():
         'ignore', '.*The property GtkAlignment:[^\s]+ is deprecated.*',
         Warning)
 
-    # Newer glib is noisy regarding deprecated signals/properties
-    # even with stable releases.
-    if is_release():
-        warnings.filterwarnings(
-            'ignore', '.* It will be removed in a future version.',
-            Warning)
-
     settings = Gtk.Settings.get_default()
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         settings.set_property("gtk-button-images", True)
         settings.set_property("gtk-menu-images", True)
     if hasattr(settings.props, "gtk_primary_button_warps_slider"):
+        # https://bugzilla.gnome.org/show_bug.cgi?id=737843
         settings.set_property("gtk-primary-button-warps-slider", True)
 
     # Make sure PyGObject includes support for foreign cairo structs
@@ -347,13 +388,12 @@ def _init_gtk():
     # blacklist some modules, simply loading can cause segfaults
     sys.modules["gtk"] = None
     sys.modules["gpod"] = None
-    sys.modules["glib"] = None
-    sys.modules["gobject"] = None
     sys.modules["gnome"] = None
 
-    from quodlibet.qltk import pygobject_version
-    if pygobject_version < (3, 9):
-        GObject.threads_init()
+    from quodlibet.qltk import pygobject_version, gtk_version
+
+    MinVersions.GTK.check(gtk_version)
+    MinVersions.PYGOBJECT.check(pygobject_version)
 
 
 def _init_gst():
@@ -457,15 +497,22 @@ def _init_gettext():
 
 
 def _init_python():
-    if sys.version_info < MinVersions.PYTHON:
-        actual = Version(sys.version_info[:3])
-        raise ImportError("Python %s required. %s found." %
-                          (MinVersions.PYTHON, actual))
+    if PY2 or is_release():
+        MinVersions.PYTHON2.check(sys.version_info)
+    else:
+        # for non release builds we allow Python3
+        MinVersions.PYTHON3.check(sys.version_info)
 
-    builtins.__dict__["print_"] = print_
-    builtins.__dict__["print_d"] = print_d
-    builtins.__dict__["print_e"] = print_e
-    builtins.__dict__["print_w"] = print_w
+    if is_osx():
+        # We build our own openssl on OSX and need to make sure that
+        # our own ca file is used in all cases as the non-system openssl
+        # doesn't use the system certs
+        util.install_urllib2_ca_file()
+
+    if is_windows():
+        # Not really needed on Windows as pygi-aio seems to work fine, but
+        # wine doesn't have certs which we use for testing.
+        util.install_urllib2_ca_file()
 
 
 def _init_formats():
@@ -492,11 +539,12 @@ def init_cli(no_translations=False):
     if not no_translations and "QUODLIBET_NO_TRANS" not in environ:
         _init_gettext()
     _init_formats()
+    _init_g()
 
     _cli_initialized = True
 
 
-def init(**kwargs):
+def init(no_translations=False, no_excepthook=False):
     """This needs to be called before any API can be used.
     Might raise in case of an error.
 
@@ -508,9 +556,9 @@ def init(**kwargs):
     if _initialized:
         return
 
-    init_cli(**kwargs)
+    init_cli(no_translations=no_translations)
     _init_gtk()
-    _init_gtk_debug()
+    _init_gtk_debug(no_excepthook=no_excepthook)
     _init_gst()
     _init_dbus()
 
@@ -593,16 +641,17 @@ def finish_first_session(app_name):
     config.set("memory", "%s_last_active_version" % app_name, const.VERSION)
 
 
-def _init_gtk_debug():
+def _init_gtk_debug(no_excepthook):
     from gi.repository import GLib
-    from quodlibet.qltk.debugwindow import ExceptionDialog
+    from quodlibet.qltk.debugwindow import excepthook
 
     print_d("Initializing debugging extensions")
 
     def _override_exceptions():
         print_d("Enabling custom exception handler.")
-        sys.excepthook = ExceptionDialog.excepthook
-    GLib.idle_add(_override_exceptions)
+        sys.excepthook = excepthook
+    if not no_excepthook:
+        GLib.idle_add(_override_exceptions)
 
     # faulthandler gives a python stacktrace on segfaults..
     try:
@@ -628,6 +677,9 @@ def _init_osx(window):
         window.set_as_osx_window(osx_app)
         osx_app.ready()
 
+    shared_app = NSApplication.sharedApplication()
+    gtk_delegate = shared_app.delegate()
+
     # Instead of quitting when the main window gets closed just hide it.
     # If the dock icon gets clicked we get
     # applicationShouldHandleReopen_hasVisibleWindows_ and show everything.
@@ -647,7 +699,9 @@ def _init_osx(window):
             GLib.idle_add(app.quit)
             return False
 
-    shared_app = NSApplication.sharedApplication()
+        def applicationDockMenu_(self, sender):
+            return gtk_delegate.applicationDockMenu_(sender)
+
     delegate = Delegate.alloc().init()
     delegate.retain()
     shared_app.setDelegate_(delegate)
