@@ -16,7 +16,7 @@ from quodlibet import util
 from quodlibet.pattern import XMLFromMarkupPattern
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.textedit import PatternEditBox
-from quodlibet.util import connect_obj
+from quodlibet.util import connect_obj, print_d
 from quodlibet.util.library import background_filter
 
 
@@ -138,6 +138,7 @@ class Browser(Gtk.Box, Filter):
         'songs-selected':
         (GObject.SignalFlags.RUN_LAST, None, (object, object)),
         'songs-activated': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'uri-received': (GObject.SignalFlags.RUN_LAST, None, (str,))
     }
 
     name = _("Library Browser")
@@ -154,9 +155,6 @@ class Browser(Gtk.Box, Filter):
     priority = 100
     """Priority in the menu list (0 is first, higher numbers come later)"""
 
-    is_empty = False
-    """Whether the browser is usable or just the dummy/disabled one"""
-
     uses_main_library = True
     """Whether the browser has the main library as source"""
 
@@ -232,7 +230,7 @@ class Browser(Gtk.Box, Filter):
 
     can_reorder = False
     """If the song list should be reorderable. In case this is True
-    every time the song list gets reorderd the whole list of songs is
+    every time the song list gets reordered the whole list of songs is
     passed to reordered().
     """
 
