@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna,
-#           2011-2013 Nick Boultbee
+#           2011-2013,2016 Nick Boultbee
 #           2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -8,13 +8,15 @@
 # published by the Free Software Foundation
 
 import os
-from cStringIO import StringIO
+
+from quodlibet.util.string import split_escape
 
 from quodlibet import browsers
 
+from quodlibet.compat import cBytesIO as StringIO
 from quodlibet import util
-from quodlibet.util.uri import URI
-from quodlibet.util.path import fsnative
+from quodlibet.util.path import fsnative, uri_to_path
+from quodlibet.util import print_d, print_e
 
 from quodlibet.qltk.browser import LibraryBrowser
 from quodlibet.qltk.properties import SongProperties
@@ -74,7 +76,7 @@ class CommandRegistry(object):
         if len(args) > argcount + optcount:
             raise CommandError("Too many arguments for %r" % name)
 
-        print_d("Running %r with params %r " % (cmd, args))
+        print_d("Running %r with params %s " % (cmd.__name__, args))
 
         try:
             return cmd(app, *args)
@@ -172,6 +174,19 @@ def _order(app, value):
             pass
 
 
+@registry.register("stop-after", args=1)
+def _stop_after(app, value):
+    po = app.player_options
+    if value == "0":
+        po.stop_after = False
+    elif value == "1":
+        po.stop_after = True
+    elif value == "t":
+        po.stop_after = not po.stop_after
+    else:
+        raise CommandError("Invalid value %r" % value)
+
+
 @registry.register("repeat", args=1)
 def _repeat(app, value):
     repeat = app.window.repeat
@@ -245,18 +260,14 @@ def _set_rating(app, value):
 def _dump_browsers(app):
     f = StringIO()
     for i, b in enumerate(browsers.browsers):
-        if not b.is_empty:
-            f.write("%d. %s\n" % (i, browsers.name(b)))
+        f.write("%d. %s\n" % (i, browsers.name(b)))
     return f.getvalue()
 
 
 @registry.register("set-browser", args=1)
 def _set_browser(app, value):
-    try:
-        browsers.get(value)
-    except ValueError:
+    if not app.window.select_browser(value, app.library, app.player):
         raise CommandError("Unknown browser %r" % value)
-    app.window.select_browser(None, value, app.library, app.player)
 
 
 @registry.register("open-browser", args=1)
@@ -332,14 +343,15 @@ def _enqueue(app, value):
 
 @registry.register("enqueue-files", args=1)
 def _enqueue_files(app, value):
-    """Enqueues comma-separated filenames or song names."""
+    """Enqueues comma-separated filenames or song names.
+    Commas in filenames should be backslash-escaped"""
 
     library = app.library
     window = app.window
     songs = []
-    for param in value.split(","):
+    for param in split_escape(value, ","):
         try:
-            song_path = URI(param).filename
+            song_path = uri_to_path(param)
         except ValueError:
             song_path = param
         if song_path in library:
@@ -397,12 +409,8 @@ def _status(app):
 
 @registry.register("song-list", args=1)
 def _song_list(app, value):
-    window = app.window
-    if value.startswith("t"):
-        value = not window.song_scroller.get_property('visible')
-    else:
-        value = value not in ['0', 'off', 'false']
-    window.song_scroller.set_property('visible', value)
+    # deprecated
+    return
 
 
 @registry.register("queue", args=1)
@@ -459,9 +467,14 @@ def _print_playing(app, fstring="<artist~album~tracknumber~title>"):
     from quodlibet.formats import AudioFile
     from quodlibet.pattern import Pattern
 
-    song = app.player.song
+    song = app.player.info
     if song is None:
         song = AudioFile({"~filename": fsnative(u"/")})
         song.sanitize()
 
     return Pattern(fstring).format(song) + "\n"
+
+
+@registry.register("uri-received", args=1)
+def _uri_received(app, uri):
+    app.browser.emit("uri-received", uri)
