@@ -1,5 +1,9 @@
 # -*- coding: utf-8 -*-
-# Constants used in various parts of QL, mostly strings.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation
+
+"""Constants used in various parts of QL, mostly strings."""
 
 import sys
 import os
@@ -8,9 +12,10 @@ import os
 class Version(tuple):
     """Represent the version of a dependency as a tuple"""
 
-    def __new__(cls, name, *args):
+    def __new__(cls, name, *args, **kwargs):
         inst = tuple.__new__(Version, args)
         inst.name = name
+        inst.message = kwargs.pop("message", "")
         return inst
 
     def human_version(self):
@@ -24,8 +29,9 @@ class Version(tuple):
 
         if self[0] == version_tuple[0] and version_tuple >= self:
             return
-        raise ImportError("%s %s required. %s found." % (
-            self.name, self, Version("", *version_tuple)))
+        message = " " + self.message if self.message else ""
+        raise ImportError("%s %s required. %s found.%s" % (
+            self.name, self, Version("", *version_tuple), message))
 
 
 class MinVersions(object):
@@ -33,17 +39,19 @@ class MinVersions(object):
 
     PYTHON2 = Version("Python2", 2, 7)
     PYTHON3 = Version("Python3", 3, 4)
-    MUTAGEN = Version("Mutagen", 1, 30)
+    MUTAGEN = Version("Mutagen", 1, 32,
+        message="Use the Quod Libet unstable PPAs/repos to get a newer "
+                "mutagen version.")
     GTK = Version("GTK+", 3, 10)
-    PYGOBJECT = Version("PyGObject", 3, 10)
+    PYGOBJECT = Version("PyGObject", 3, 12)
     GSTREAMER = Version("GStreamer", 1, 0)
 
 
-VERSION_TUPLE = Version("", 3, 6, 2)
+VERSION_TUPLE = Version("", 3, 7, 0)
 VERSION = str(VERSION_TUPLE)
 
 # entry point for the user guide / wiki
-BRANCH_NAME = "quodlibet-3.6"
+BRANCH_NAME = "quodlibet-3.7"
 DOCS_BASE_URL = "https://quodlibet.readthedocs.org/en/%s"
 DOCS_LATEST = DOCS_BASE_URL % "latest"
 DOCS_BASE_URL %= BRANCH_NAME if BRANCH_NAME != "master" else "latest"
@@ -57,7 +65,7 @@ EMAIL = os.environ.get("EMAIL", "quodlibet@lists.sacredchao.net")
 # Displayed as registered / help email address
 SUPPORT_EMAIL = "quod-libet-development@googlegroups.com"
 
-MAIN_AUTHORS = """\
+MAIN_AUTHORS = u"""\
 Joe Wreschnig
 Michael Urman
 Iñigo Serna
@@ -67,9 +75,9 @@ Nick Boultbee""".split("\n")
 
 # about dialog, --version etc.
 WEBSITE = "https://quodlibet.readthedocs.org/"
-COPYRIGHT = """Copyright © 2004-2016 %s...""" % ", ".join(MAIN_AUTHORS)
+COPYRIGHT = u"""Copyright © 2004-2016 %s...""" % u", ".join(MAIN_AUTHORS)
 
-AUTHORS = sorted("""\
+AUTHORS = sorted(u"""\
 Alexandre Passos
 Alexey Bobyakov
 Alex Geoffrey Smith
@@ -83,6 +91,7 @@ Bastian Kleineidam
 Bastien Gorissen
 Benjamin Boutier
 Ben Zeigler
+Bernd Wechner
 Bruno Bergot
 Carlo Teubner
 Christine Spang
@@ -101,6 +110,7 @@ Guillaume Chazarain
 Hans Scholze
 Iñigo Serna
 Jacob Lee
+Jakob Gahde
 Jan Arne Petersen
 Jan Path
 Javier Kohen
@@ -143,10 +153,11 @@ Tomasz Torcz
 Tshepang Lekhonkhobe
 Türerkan İnce
 Vasiliy Faronov
+Victoria Hayes
 Zack Weinberg
 """.strip().split("\n"))
 
-TRANSLATORS = sorted("""
+TRANSLATORS = sorted(u"""
 Alexandre Passos (pt)
 Andreas Bertheussen (nb)
 Anton Shestakov (ru)
@@ -174,6 +185,7 @@ Johám-Luís Miguéns Vila (es, gl, gl_ES, eu, pt)
 Jonas Slivka (lt)
 Joshua Kwan (fr)
 Luca Baraldi (it)
+Ludovic Druette (fr)
 Lukáš Lalinský (sk)
 Mathieu Morey (fr)
 Michal Nowikowski (pl)
@@ -200,7 +212,7 @@ Yasushi Iwata (ja)
 Сергей Федосеев (ru)
 """.strip().splitlines())
 
-ARTISTS = sorted("""\
+ARTISTS = sorted(u"""\
 Tobias
 Jakub Steiner
 Fabien Devaux
