@@ -12,12 +12,10 @@ import re
 import ctypes
 import ctypes.util
 import sys
-import traceback
 import unicodedata
 import threading
 import subprocess
 import webbrowser
-import contextlib
 
 # Windows doesn't have fcntl, just don't lock for now
 try:
@@ -26,19 +24,40 @@ try:
 except ImportError:
     fcntl = None
 
-from quodlibet.compat import reraise as py_reraise, urlparse
-from quodlibet.util.path import iscommand, is_fsnative
+from quodlibet.compat import reraise as py_reraise, PY2, text_type, \
+    iteritems, reduce
+from quodlibet.util.path import iscommand
 from quodlibet.util.string.titlecase import title
 
 from quodlibet.const import SUPPORT_EMAIL, COPYRIGHT
-from quodlibet.util.dprint import print_d, print_
+from quodlibet.util.dprint import print_d, print_, print_e, print_w, print_exc
 from .misc import environ, argv, cached_func, get_locale_encoding, \
     get_fs_encoding
-from .environment import *
+from .environment import is_plasma, is_unity, is_enlightenment, \
+    is_linux, is_windows, is_wine, is_osx, is_py2exe, is_py2exe_console, \
+    is_py2exe_window
+from .enum import enum
 
 
 # pyflakes
-environ, argv, cached_func, get_locale_encoding, get_fs_encoding
+environ, argv, cached_func, get_locale_encoding, get_fs_encoding, enum,
+print_w, print_exc, is_plasma, is_unity, is_enlightenment,
+is_linux, is_windows, is_wine, is_osx, is_py2exe, is_py2exe_console,
+is_py2exe_window
+
+
+if PY2:
+    def gdecode(s):
+        """Returns unicode for the glib text type"""
+
+        assert isinstance(s, bytes)
+        return s.decode("utf-8")
+else:
+    def gdecode(s):
+        """Returns unicode for the glib text type"""
+
+        assert isinstance(s, text_type)
+        return s
 
 
 class InstanceTracker(object):
@@ -217,17 +236,21 @@ def italic(string):
     return "<i>%s</i>" % string
 
 
-def parse_time(timestr, err=(ValueError, re.error)):
+def parse_time(timestr, err=object()):
     """Parse a time string in hh:mm:ss, mm:ss, or ss format."""
+
     if timestr[0:1] == "-":
         m = -1
         timestr = timestr[1:]
     else:
         m = 1
+
     try:
         return m * reduce(lambda s, a: s * 60 + int(a),
                           re.split(r":|\.", timestr), 0)
-    except err:
+    except (ValueError, re.error):
+        if err is None:
+            raise
         return 0
 
 
@@ -413,7 +436,8 @@ def _split_numeric_sortkey(s, limit=10,
     it can be used for human sorting. Also removes all extra whitespace."""
     result = reg(s)
     if not result or not limit:
-        return (join(s.split()),)
+        text = join(s.split())
+        return (text,) if text else tuple()
     else:
         start, end = result.span()
         return (
@@ -423,10 +447,12 @@ def _split_numeric_sortkey(s, limit=10,
 
 
 def human_sort_key(s, normalize=unicodedata.normalize):
-    if not isinstance(s, unicode):
+    if not s:
+        return ()
+    if not isinstance(s, text_type):
         s = s.decode("utf-8")
     s = normalize("NFD", s.lower())
-    return s and _split_numeric_sortkey(s)
+    return _split_numeric_sortkey(s)
 
 
 def website(site):
@@ -512,10 +538,12 @@ def tagsplit(tag):
         return [tag]
 
 
-def pattern(pat, cap=True, esc=False):
+def pattern(pat, cap=True, esc=False, markup=False):
     """Return a 'natural' version of the pattern string for human-readable
-    bits. Assumes all tags in the pattern are present."""
-    from quodlibet.pattern import Pattern, XMLFromPattern
+    bits. Assumes all tags in the pattern are present.
+    """
+
+    from quodlibet.pattern import Pattern, XMLFromPattern, XMLFromMarkupPattern
 
     class Fakesong(dict):
         cap = False
@@ -525,13 +553,20 @@ def pattern(pat, cap=True, esc=False):
 
         def list(self, key):
             return [tag(k, self.cap) for k in tagsplit(key)]
-        list_seperate = list
-        __call__ = comma
+        list_separate = list
+
+        def __call__(self, tag, *args):
+            return 0 if '~#' in tag[:2] else self.comma(tag)
 
     fakesong = Fakesong({'filename': tag('filename', cap)})
     fakesong.cap = cap
     try:
-        p = (esc and XMLFromPattern(pat)) or Pattern(pat)
+        if markup:
+            p = XMLFromMarkupPattern(pat)
+        elif esc:
+            p = XMLFromPattern(pat)
+        else:
+            p = Pattern(pat)
     except ValueError:
         return _("Invalid pattern")
 
@@ -566,21 +601,10 @@ def fver(tup):
     return ".".join(map(str, tup))
 
 
-def uri_is_valid(uri):
-    return bool(urlparse(uri)[0])
-
-
 def make_case_insensitive(filename):
     return "".join(["[%s%s]" % (c.lower(), c.upper()) for c in filename])
 
 
-def print_exc(limit=None, file=None):
-    """A wrapper preventing crashes on broken pipes in print_exc."""
-    if not file:
-        file = sys.stderr
-    print_(traceback.format_exc(limit=limit), output=file)
-
-
 class DeferredSignal(object):
     """A wrapper for connecting functions to signals.
 
@@ -589,7 +613,7 @@ class DeferredSignal(object):
     priority and prevents multiple calls from being inserted in the
     mainloop at a time, greatly improving responsiveness in some places.
 
-    When the target function will finally be called the arguments passed
+    When the target function is finally called, the arguments passed
     are the last arguments passed to DeferredSignal.
 
     `priority` defaults to GLib.PRIORITY_DEFAULT
@@ -628,8 +652,8 @@ class DeferredSignal(object):
                 timeout, f, priority=priority)
 
     @property
-    def im_self(self):
-        return self.func.im_self
+    def __self__(self):
+        return self.func.__self__
 
     @property
     def __code__(self):
@@ -639,6 +663,13 @@ class DeferredSignal(object):
     def __closure__(self):
         return self.func.__closure__
 
+    def call(self, *args):
+        """Force a call"""
+
+        self.abort()
+        self.args = args
+        self._wrap()
+
     def abort(self):
         """Abort any queued up calls.
 
@@ -693,8 +724,8 @@ def _connect_destroy(sender, func, detailed_signal, handler, *args, **kwargs):
     to the bound method and the bound to object doesn't get GCed.
     """
 
-    if hasattr(handler, "im_self"):
-        obj = handler.im_self
+    if hasattr(handler, "__self__"):
+        obj = handler.__self__
     else:
         # XXX: get the "self" var of the enclosing scope.
         # Used for nested functions which ref the object but aren't methods.
@@ -747,7 +778,7 @@ def sanitize_tags(tags, stream=False):
     """
 
     san = {}
-    for key, value in tags.iteritems():
+    for key, value in iteritems(tags):
         key = key.lower()
         key = {"location": "website"}.get(key, key)
 
@@ -886,56 +917,16 @@ def gi_require_versions(name, versions):
 
     import gi
 
+    error = None
     for version in versions:
         try:
             gi.require_version(name, version)
         except ValueError as e:
-            pass
+            error = e
         else:
             return version
     else:
-        raise e
-
-
-@contextlib.contextmanager
-def atomic_save(filename, suffix, mode):
-    """Try to replace the content of a file in the safest way possible.
-
-    * filename+suffix will be created during the process.
-    * On UNIX this operation is atomic, on Windows it is not.
-
-    with atomic_save("config.cfg", ".tmp", "wb") as f:
-        f.write(data)
-
-    Can raise.
-    """
-
-    assert is_fsnative(filename)
-
-    temp_filename = filename + suffix
-    fileobj = open(temp_filename, "wb")
-    try:
-        if fcntl is not None:
-            fcntl.flock(fileobj.fileno(), fcntl.LOCK_EX)
-
-        yield fileobj
-
-        fileobj.flush()
-        os.fsync(fileobj.fileno())
-
-        # No atomic rename on windows
-        if os.name == "nt":
-            fileobj.close()
-            try:
-                os.remove(filename)
-            except EnvironmentError:
-                pass
-
-        os.rename(temp_filename, filename)
-    finally:
-        if fcntl is not None:
-            fcntl.flock(fileobj.fileno(), fcntl.LOCK_UN)
-        fileobj.close()
+        raise error
 
 
 def load_library(names, shared=True):
@@ -1118,41 +1109,6 @@ def re_escape(string, BAD="/.^$*+-?{,\\[]|()<>#=!:"):
     return type(string)().join(map(needs_escape, string))
 
 
-def enum(cls):
-    """Class decorator for enum types::
-
-        @enum
-        class SomeEnum(int):
-            FOO = 0
-            BAR = 1
-
-    Result is an int subclass and all attributes are instances of it.
-    """
-
-    type_ = cls.__bases__[0]
-
-    d = dict(cls.__dict__)
-    new_type = type(cls.__name__, (type_,), d)
-    new_type.__module__ = cls.__module__
-
-    map_ = {}
-    for key, value in d.iteritems():
-        if key.upper() == key and isinstance(value, type_):
-            value_instance = new_type(value)
-            setattr(new_type, key, value_instance)
-            map_[value] = key
-
-    def repr_(self):
-        if self in map_:
-            return "%s.%s" % (type(self).__name__, map_[self])
-        else:
-            return "%s(%s)" % (type(self).__name__, self)
-
-    setattr(new_type, "__repr__", repr_)
-
-    return new_type
-
-
 def set_process_title(title):
     """Sets process name as visible in ps or top. Requires ctypes libc
     and is almost certainly *nix-only. See issue 736
@@ -1194,3 +1150,59 @@ def reraise(tp, value, tb=None):
     if tb is None:
         tb = sys.exc_info()[2]
     py_reraise(tp, value, tb)
+
+
+def get_module_dir(module=None):
+    """Returns the absolute path of a module. If no module is given
+    the one this is called from is used.
+    """
+
+    if module is None:
+        file_path = sys._getframe(1).f_globals["__file__"]
+    else:
+        file_path = getattr(module, "__file__")
+    if is_windows():
+        file_path = file_path.decode(sys.getfilesystemencoding())
+    return os.path.dirname(os.path.realpath(file_path))
+
+
+def get_ca_file():
+    """A path to a CA file or None.
+
+    Depends whether we use certifi or the system trust store
+    on the current platform.
+    """
+
+    if is_linux():
+        return None
+
+    import certifi
+
+    return os.path.join(get_module_dir(certifi), "cacert.pem")
+
+
+def install_urllib2_ca_file():
+    """Makes urllib2.urlopen and urllib2.build_opener use the ca file
+    returned by get_ca_file()
+    """
+
+    try:
+        import ssl
+    except ImportError:
+        return
+
+    import urllib2
+
+    base = urllib2.HTTPSHandler
+
+    class MyHandler(base):
+
+        def __init__(self, debuglevel=0, context=None):
+            ca_file = get_ca_file()
+            if context is None and ca_file is not None:
+                context = ssl.create_default_context(
+                    purpose=ssl.Purpose.SERVER_AUTH,
+                    cafile=ca_file)
+            base.__init__(self, debuglevel, context)
+
+    urllib2.HTTPSHandler = MyHandler
